#include "project.h"
#include "stdio.h"
#include "tty_utils.h"
#include "pwm_3phase.h"

#define LED_ON 0u
#define LED_OFF 1u

char cmd[STR_BUF_LEN + 1] ;

void do_period(char *str) ;
void do_status(char *str) ;
void do_reset(char *str) ;
void do_enable(char *str) ;
void do_disable(char *str) ;
void do_help(char *str) ;
void do_command(char *str) ;


void init_hardware(void)
{
    CyGlobalIntEnable; /* Enable global interrupts. */
    
    tty_init() ;
    
    pwm_3phase_start() ;
}

int main(void)
{
    init_hardware() ;
    
    splash("=== 3Phase PWM Test ===") ;
    
    prompt() ;
    
    for(;;)
    {
        if (get_line()) {
            do_command(str) ;
            prompt() ;
        }
        if (pwm_3phase_flag == 1) { /* tc deteced */
            /* do something for pwm_3pahse tc */
            pwm_3phase_flag = 0 ;
        }
    }
}

void do_command(char *str) 
{
    sscanf(str, "%s", cmd) ;
    if (strcmp(cmd, "period") == 0) {
        do_period(str) ;
    } else if (strcmp(cmd, "status") == 0) {
        do_status(str) ;
    } else if (strcmp(cmd, "reset") == 0) {
        do_reset(str) ;
    } else if (strcmp(cmd, "enable") == 0) {
        do_enable(str) ;
    } else if (strcmp(cmd, "disable") == 0) {
        do_disable(str) ;
    } else if (strcmp(cmd, "help") == 0) {
        do_help(str) ;
    } else {
        print("Unknown command ") ;
        print(cmd) ;
        print("\n\r") ;
        do_help(str) ;
    }
}

void do_period(char *str) 
{
    uint16_t period ;
    sscanf(str, "%*s %hd", &period) ;
    pwm_3phase_set_period(period) ;
}

void do_reset(char *str) 
{
    (void) str ;
    pwm_3phase_reset() ;
}

void do_enable(char *str) 
{
    (void) str ;
    pwm_3phase_enable() ;
}

void do_disable(char *str) 
{
    (void) str ;
    pwm_3phase_disable() ;
}

void do_status(char *str)
{
    (void) str ;
    uint16_t period ;
    uint16_t compare1, compare2 ;
    print("=== status ===\n\r") ;
    period = PWM_1_ReadPeriod() ;
    snprintf(str, STR_BUF_LEN, "period: %d\n\r", period) ;
    print(str) ;
    compare1 = PWM_1_ReadCompare1() ;
    compare2 = PWM_1_ReadCompare2() ;
    print("PWM 1 : ") ;
    snprintf(str, STR_BUF_LEN, "compare1: %d compare2: %d\n\r", 
        compare1, compare2) ;
    print(str) ;

    compare1 = PWM_2_ReadCompare1() ;
    compare2 = PWM_2_ReadCompare2() ;
    print("PWM 2 : ") ;
    snprintf(str, STR_BUF_LEN, "compare1: %d compare2: %d\n\r", 
        compare1, compare2) ;
    print(str) ;
    
    compare1 = PWM_3_ReadCompare1() ;
    compare2 = PWM_3_ReadCompare2() ;
    print("PWM 3 : ") ;
    snprintf(str, STR_BUF_LEN, "compare1: %d compare2: %d\n\r", 
        compare1, compare2) ;
    print(str) ;
    print("=============\n\r") ;  
}

void do_help(char *str)
{
    (void) str ;
    print("=== commands ===\n\r") ;
    print("period <value> : set PWM period to <value>\n\r") ;
    print("status: resport current configuration\n\r") ;
    print("reset : reset PWMs\n\r") ;
    print("enable : enable PWMs\n\r") ;
    print("disable : diable PWMs\n\r") ;
    print("=================\n\r") ;
}
  

