#include "project.h"
#include "pwm_3phase.h"

#define PWM_3P_RESET  0x01
#define PWM_3P_ENABLE 0x02

volatile int pwm_3phase_flag = 0 ;

CY_ISR(pwm_3phase_isr) 
{
    pwm_3phase_flag = 1 ;
    PWM_3_ReadStatusRegister() ;
    LED_Write( ~LED_Read() ) ; /* toggle LED */
}

void pwm_3phase_start(void) 
{
    pwm_3phase_init() ;
    pwm_3phase_enable() ;
    isr_1_StartEx(pwm_3phase_isr) ;
}

void pwm_3phase_init(void) 
{
    uint16_t period ;
    PWM_1_Init() ;
    PWM_2_Init() ;
    PWM_3_Init() ;
    period = PWM_1_ReadPeriod() ;
    
    pwm_3phase_set_period(PWM_1_ReadPeriod()) ;
}

void pwm_3phase_set_period(uint16_t period)
{
    uint16_t real_period, period3 ;
    uint16_t tmp ;
    uint16_t delta1 = 0 ;
    uint16_t delta2 = 0 ;
    
    real_period = period + 1 ;
    period3 = (real_period) / 3 ;
    PWM_1_WritePeriod(period) ;
    PWM_2_WritePeriod(period) ;
    PWM_3_WritePeriod(period) ;
    
    switch(real_period - (period3 * 3)) {
    case 0: break ;
    case 1: delta1 = 1 ; break ;
    case 2: delta1 = 1 ; delta2 = 1 ; break ;
    }
    
    tmp = period ;
    PWM_1_WriteCompare2(tmp) ;
    tmp = real_period - (period3 + delta1) ;
    PWM_1_WriteCompare1(tmp) ;
    
    tmp = tmp - 1 ;
    PWM_2_WriteCompare2(tmp) ;
    tmp = real_period - (2 * period3 + delta1 + delta2) ;
    PWM_2_WriteCompare1(tmp) ;
    
    tmp = tmp - 1 ;
    PWM_3_WriteCompare2(tmp) ;
    PWM_3_WriteCompare1(0) ;
    
    pwm_3phase_reset() ;
    pwm_3phase_enable() ;
}

uint16_t pwm_3phase_get_period(void) 
{
    uint16_t period ;
    period = PWM_1_ReadPeriod() ;
    return( period ) ;
}

void pwm_3phase_reset(void) 
{
    Control_Reg_1_Write(PWM_3P_RESET) ;
    CyDelay(1) ;
    Control_Reg_1_Write(0) ;
}

void pwm_3phase_enable(void) 
{
    Control_Reg_1_Write(PWM_3P_ENABLE) ;
}

void pwm_3phase_disable(void) 
{
    Control_Reg_1_Write(0) ;
}